@@ -22,7 +22,6 @@
 
 #include "KSMainWindow.h"
 
-#include "Config.h"
 #include "settings.h"
 #include "SettingsDialog/SettingsDialog.h"
 #include "SettingsDialog/GeneralOptionsPage.h"
@@ -80,6 +79,10 @@ KSMainWindow::KSMainWindow(Platform::GrabModes theGrabModes, Platform::ShutterMo
     mScreenRecorderToolsMenu(new QMenu(this)),
     mExportMenu(new ExportMenu(this)),
     mShutterModes(theShutterModes)
+#ifdef KIMAGEANNOTATOR_FOUND
+    ,mAnnotateButton(new QToolButton(this))
+    ,mAnnotatorActive(false)
+#endif
 {
     // before we do anything, we need to set a window property
     // that skips the close/hide window animation on kwin. this
@@ -152,6 +155,31 @@ void KSMainWindow::init()
     mConfigureButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     mDialogButtonBox->addButton(mConfigureButton, QDialogButtonBox::ResetRole);
 
+#ifdef KIMAGEANNOTATOR_FOUND
+    mAnnotateButton->setText(i18n("Annotate"));
+    mAnnotateButton->setToolTip(i18n("Add annotation to the screenshot"));
+    mAnnotateButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+    mAnnotateButton->setIcon(QIcon::fromTheme(QStringLiteral("document-edit")));
+    connect(mAnnotateButton, &QToolButton::pressed, this, [this] {
+
+        if (mAnnotatorActive) {
+            mKSWidget->hideAnnotator();
+            mAnnotateButton->setText(i18n("Annotate"));
+        } else {
+            mKSWidget->showAnnotator();
+            mAnnotateButton->setText(i18n("Annotation done"));
+        }
+        mAnnotatorActive = !mAnnotatorActive;
+
+        mToolsButton->setEnabled(!mAnnotatorActive);
+        mSendToButton->setEnabled(!mAnnotatorActive);
+        mClipboardButton->setEnabled(!mAnnotatorActive);
+        mSaveButton->setEnabled(!mAnnotatorActive);
+    });
+
+    mDialogButtonBox->addButton(mAnnotateButton, QDialogButtonBox::ActionRole);
+#endif
+
     KGuiItem::assign(mToolsButton, KGuiItem(i18n("Tools")));
     mToolsButton->setIcon(QIcon::fromTheme(QStringLiteral("tools"),
                                            QIcon::fromTheme(QStringLiteral("application-menu"))));
@@ -458,6 +486,12 @@ void KSMainWindow::imageCopied()
                       KMessageWidget::Information);
 }
 
+void KSMainWindow::screenshotFailed()
+{
+    showInlineMessage(i18n("Could not take a screenshot. Please report this bug here: <a href=\"https://bugs.kde.org/enter_bug.cgi?product=Spectacle\">create a spectacle bug</a>"),
+                      KMessageWidget::Warning);
+}
+
 void KSMainWindow::showPreferencesDialog()
 {
     if (KConfigDialog::showDialog(QStringLiteral("settings"))) {
