@@ -30,6 +30,8 @@
 #include <QDBusUnixFileDescriptor>
 #include <QDBusPendingCall>
 #include <QFutureWatcher>
+#include <QScreen>
+#include <QGuiApplication>
 
 #include <array>
 
@@ -73,6 +75,30 @@ static QImage readImage(int thePipeFd)
     return lImage;
 }
 
+static QVector<QImage> readImages(int thePipeFd)
+{
+    QByteArray lContent;
+    if (readData(thePipeFd, lContent) != 0) {
+        close(thePipeFd);
+        return QVector<QImage>();
+    }
+    close(thePipeFd);
+
+    QDataStream lDataStream(lContent);
+    lDataStream.setVersion(QDataStream::Qt_DefaultCompiledVersion);
+
+    QImage lImage;
+    QVector<QImage> imgs;
+    while (!lDataStream.atEnd()){
+        lDataStream >> lImage;
+        if (!lImage.isNull()) {
+            imgs << lImage;
+        }
+    }
+
+    return imgs;
+}
+
 /* -- General Plumbing ------------------------------------------------------------------------- */
 
 PlatformKWinWayland::PlatformKWinWayland(QObject *parent) :
@@ -84,15 +110,6 @@ QString PlatformKWinWayland::platformName() const
     return QStringLiteral("KWinWayland");
 }
 
-Platform::GrabModes PlatformKWinWayland::supportedGrabModes() const
-{
-    Platform::GrabModes lSupportedModes({ GrabMode::AllScreens, GrabMode::WindowUnderCursor });
-    if (QApplication::screens().count() > 1) {
-        lSupportedModes |= Platform::GrabMode::CurrentScreen;
-    }
-    return lSupportedModes;
-}
-
 static std::array<int, 3> s_plasmaVersion = {-1, -1, -1};
 
 std::array<int, 3> findPlasmaMinorVersion () {
@@ -138,22 +155,77 @@ std::array<int, 3> findPlasmaMinorVersion () {
     return s_plasmaVersion;
 }
 
+Platform::GrabModes PlatformKWinWayland::supportedGrabModes() const
+{
+    Platform::GrabModes lSupportedModes({ Platform::GrabMode::AllScreens, GrabMode::WindowUnderCursor });
+    QList<QScreen *> screens = QApplication::screens();
+
+    // TODO remove sometime after Plasma 5.21 is released
+    // We can handle rectangular selection one one screen not scale factor
+    // on Plasma < 5.21
+    if (screenshotScreensAvailable() || (screens.count() == 1 && screens.first()->devicePixelRatio() == 1)) {
+        lSupportedModes |= Platform::GrabMode::PerScreenImageNative;
+    }
+
+    // TODO remove sometime after Plasma 5.20 is released
+    auto plasmaVersion = findPlasmaMinorVersion();
+    if (plasmaVersion.at(0) != -1 && (plasmaVersion.at(0) != 5 || (plasmaVersion.at(1) >= 20))) {
+        lSupportedModes |= Platform::GrabMode::AllScreensScaled;
+    }
+
+    if (screens.count() > 1) {
+        lSupportedModes |= Platform::GrabMode::CurrentScreen;
+    }
+    return lSupportedModes;
+}
+
+bool PlatformKWinWayland::screenshotScreensAvailable() const
+{
+    // TODO remove sometime after Plasma 5.21 is released
+    auto plasmaVersion = findPlasmaMinorVersion();
+    // Screenshot screenshotScreens dbus interface requires Plasma 5.21
+    if (plasmaVersion.at(0) != -1 && (plasmaVersion.at(0) != 5 || (plasmaVersion.at(1) >= 21 || (plasmaVersion.at(1) == 20 && plasmaVersion.at(2) >= 80)))) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
 Platform::ShutterModes PlatformKWinWayland::supportedShutterModes() const
 {
     // TODO remove sometime after Plasma 5.20 is released
     auto plasmaVersion = findPlasmaMinorVersion();
-    if (plasmaVersion.at(0) != -1 && (plasmaVersion.at(0) != 5 || (plasmaVersion.at(1) >= 20 || (plasmaVersion.at(1) == 19 && plasmaVersion.at(2) >= 80)))) {
-        return { ShutterMode::Immediate | ShutterMode::OnClick };
+    if (plasmaVersion.at(0) != -1 && (plasmaVersion.at(0) != 5 || (plasmaVersion.at(1) >= 20))) {
+        return { ShutterMode::Immediate };
     } else {
         return { ShutterMode::OnClick };
     }
 }
 
-void PlatformKWinWayland::doGrab(ShutterMode theShutterMode, GrabMode theGrabMode, bool theIncludePointer, bool theIncludeDecorations)
+void PlatformKWinWayland::doGrab(ShutterMode /* theShutterMode */, GrabMode theGrabMode, bool theIncludePointer, bool theIncludeDecorations)
 {
     switch(theGrabMode) {
-    case GrabMode::AllScreens: {
-        doGrabHelper(QStringLiteral("screenshotFullscreen"), theIncludePointer);
+    case GrabMode::AllScreens:
+        doGrabHelper(QStringLiteral("screenshotFullscreen"), theIncludePointer, true);
+        return;
+    case GrabMode::AllScreensScaled:
+        doGrabHelper(QStringLiteral("screenshotFullscreen"), theIncludePointer, false);
+        return;
+
+    case GrabMode::PerScreenImageNative:
+    {
+        QList<QScreen *> screens = QGuiApplication::screens();
+        QStringList screenNames;
+        for (const auto screen : screens) {
+            screenNames << screen->name();
+        }
+        if (screenshotScreensAvailable()) {
+            doGrabImagesHelper(QStringLiteral("screenshotScreens"), screenNames, theIncludePointer, true);
+        } else {
+            // TODO remove sometime after Plasma 5.21 is released
+            // Use the dbus call screenshotFullscreen to get a single screen screenshot and treat it as a list of images
+            doGrabImagesHelper(QStringLiteral("screenshotFullscreen"), theIncludePointer, true);
+        }
         return;
     }
     case GrabMode::CurrentScreen: {
@@ -185,18 +257,43 @@ void PlatformKWinWayland::startReadImage(int theReadPipe)
         [lWatcher, this] () {
             lWatcher->deleteLater();
             const QImage lImage = lWatcher->result();
-            emit newScreenshotTaken(QPixmap::fromImage(lImage));
+            if (lImage.isNull()) {
+                newScreenshotFailed();
+            } else {
+                newScreenshotTaken(QPixmap::fromImage(lImage));
+            }
         }
     );
     lWatcher->setFuture(QtConcurrent::run(readImage, theReadPipe));
 }
 
-template <typename ArgType>
-void PlatformKWinWayland::callDBus(const QString &theGrabMethod, ArgType theArgument, int theWriteFile)
+void PlatformKWinWayland::startReadImages(int theReadPipe)
+{
+    auto lWatcher = new QFutureWatcher<QVector<QImage>>(this);
+    QObject::connect(lWatcher, &QFutureWatcher<QVector<QImage>>::finished, this,
+        [lWatcher, this] () {
+            lWatcher->deleteLater();
+            auto result = lWatcher->result();
+            if (result.isEmpty()) {
+                newScreenshotFailed();
+            } else {
+                newScreensScreenshotTaken(result);
+            }
+        }
+    );
+    lWatcher->setFuture(QtConcurrent::run(readImages, theReadPipe));
+}
+
+template <typename ... ArgType>
+void PlatformKWinWayland::callDBus(const QString &theGrabMethod, int theWriteFile, ArgType ... arguments)
 {
     QDBusInterface lInterface(QStringLiteral("org.kde.KWin"), QStringLiteral("/Screenshot"), QStringLiteral("org.kde.kwin.Screenshot"));
-    QDBusPendingCall pcall = lInterface.asyncCall(theGrabMethod, QVariant::fromValue(QDBusUnixFileDescriptor(theWriteFile)), theArgument);
+    QDBusPendingCall pcall = lInterface.asyncCall(theGrabMethod, QVariant::fromValue(QDBusUnixFileDescriptor(theWriteFile)), arguments...);
+    checkDbusPendingCall(pcall);
+}
 
+void PlatformKWinWayland::checkDbusPendingCall(QDBusPendingCall pcall)
+{
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(pcall, this);
     QObject::connect(watcher, &QDBusPendingCallWatcher::finished,
                      this, [this](QDBusPendingCallWatcher* watcher) {
@@ -209,8 +306,8 @@ void PlatformKWinWayland::callDBus(const QString &theGrabMethod, ArgType theArgu
     });
 }
 
-template <typename ArgType>
-void PlatformKWinWayland::doGrabHelper(const QString &theGrabMethod, ArgType theArgument)
+template <typename ... ArgType>
+void PlatformKWinWayland::doGrabHelper(const QString &theGrabMethod, ArgType ... arguments)
 {
     int lPipeFds[2];
     if (pipe2(lPipeFds, O_CLOEXEC|O_NONBLOCK) != 0) {
@@ -218,8 +315,23 @@ void PlatformKWinWayland::doGrabHelper(const QString &theGrabMethod, ArgType the
         return;
     }
 
-    callDBus(theGrabMethod, theArgument, lPipeFds[1]);
+    callDBus(theGrabMethod, lPipeFds[1], arguments...);
     startReadImage(lPipeFds[0]);
 
     close(lPipeFds[1]);
 }
+
+template <typename ... ArgType>
+void PlatformKWinWayland::doGrabImagesHelper(const QString &theGrabMethod, ArgType ... arguments)
+{
+    int lPipeFds[2];
+    if (pipe2(lPipeFds, O_CLOEXEC|O_NONBLOCK) != 0) {
+        emit newScreenshotFailed();
+        return;
+    }
+
+    callDBus(theGrabMethod, lPipeFds[1], arguments...);
+    startReadImages(lPipeFds[0]);
+
+    close(lPipeFds[1]);
+}
